@@ -17,11 +17,12 @@
  */
 
 #ifndef _ABOUTDIALOG_H
-#define	_ABOUTDIALOG_H
+#define   _ABOUTDIALOG_H
 
 #include <QDialog>
 #include <QWidget>
 #include "ui_aboutDialog.h"
+#include "config.h"
 
 class AboutDialog;
 
@@ -30,8 +31,12 @@ class AboutDialog : public QDialog, public Ui::aboutDialog
 public:
    AboutDialog(QWidget* parent=0)
            : QDialog(parent)
-           { setupUi(this); }
+   {
+      setupUi(this);
+			// Replaces the "%1" from the .ui with version number.
+      textBrowser->setHtml( textBrowser->toHtml().arg(VERSIONSTRING) );
+   }
 };
 
-#endif	/* _ABOUTDIALOG_H */
+#endif   /* _ABOUTDIALOG_H */
 
