@@ -33,14 +33,8 @@ BrewDayScrollWidget::BrewDayScrollWidget(QWidget* parent) : QWidget(parent), Obs
    setupUi(this);
    recObs = 0;
 
-   // HAVE to do this since apparently the stackedWidget NEEDS at least 1
-   // widget at all times.
-   //stackedWidget->insertWidget(0, new InstructionWidget(stackedWidget) );
-   //stackedWidget->widget(0)->setVisible(false);
-   //stackedWidget->removeWidget(stackedWidget->widget(1));
-
-   //connect( listWidget, SIGNAL(currentRowChanged(int)), stackedWidget, SLOT(setCurrentIndex(int)) );
    connect( listWidget, SIGNAL(currentRowChanged(int)), this, SLOT(showInstruction(int)) );
+   connect( plainTextEdit, SIGNAL(textChanged()), this, SLOT(saveInstruction()) );
    connect( pushButton_insert, SIGNAL(clicked()), this, SLOT(insertInstruction()) );
    connect( pushButton_remove, SIGNAL(clicked()), this, SLOT(removeSelectedInstruction()) );
    connect( pushButton_up, SIGNAL(clicked()), this, SLOT(pushInstructionUp()) );
@@ -50,6 +44,14 @@ BrewDayScrollWidget::BrewDayScrollWidget(QWidget* parent) : QWidget(parent), Obs
    doc = new QWebView();
 }
 
+void BrewDayScrollWidget::saveInstruction()
+{
+   // Need to disable notification to avoid a possible infinite loop.
+   recObs->disableNotification();
+   recObs->getInstruction( listWidget->currentRow() )->setDirections( plainTextEdit->toPlainText() );
+   recObs->reenableNotification();
+}
+
 void BrewDayScrollWidget::showInstruction(int insNdx)
 {
    if( recObs == 0 )
@@ -105,8 +107,14 @@ void BrewDayScrollWidget::pushInstructionDown()
       return;
    
    int row = listWidget->currentRow();
+
    if( row >= listWidget->count() )
       return;
+
+   // This happens when the user is bouncing on the Step Down button but no
+   // instruction is selected.
+   if ( row < 0 )
+      return;
    
    recObs->swapInstructions(row, row+1);
    listWidget->setCurrentRow(row+1);
@@ -130,7 +138,7 @@ QString BrewDayScrollWidget::getCSS()
    return css;
 }
 
-QString BrewDayScrollWidget::buildTitleTable()
+QString BrewDayScrollWidget::buildTitleTable(bool includeImage)
 {
    QString header;
    QString body;
@@ -142,55 +150,55 @@ QString BrewDayScrollWidget::buildTitleTable()
 
    body   = "<body>";
    body += QString("<h1>%1</h1>").arg(recObs->getName());
-   body += QString("<img src=\"%1\" />").arg("qrc:/images/title.svg");
+   if ( includeImage )
+      body += QString("<img src=\"%1\" />").arg("qrc:/images/title.svg");
 
    // Build the top table
    // Build the first row: Style and Date
    body += "<table id=\"title\">";
    body += QString("<tr><td class=\"left\">%1</td>")
-		   .arg(tr("Style"));
+         .arg(tr("Style"));
    body += QString("<td class=\"value\">%1</td>")
            .arg(recObs->getStyle()->getName());
    body += QString("<td class=\"right\">%1</td>")
-		   .arg(tr("Date"));
+         .arg(tr("Date"));
    body += QString("<td class=\"value\">%1</td></tr>")
            .arg(QDate::currentDate().toString());
 
-   // second row:  boil time and efficiency.  I think there is something wrong w/ the call to getBoilTime_min(), because it returns
-   // 0 a lot.  I need to get this figured out.
-      body += QString("<tr><td class=\"left\">%1</td><td class=\"value\">%2</td><td class=\"right\">%3</td><td class=\"value\">%4</td></tr>")
-    		  .arg(tr("Boil Time"))
-              .arg(Brewtarget::displayAmount(recObs->getBoilTime_min(),Units::minutes))
-              .arg(tr("Efficiency"))
-              .arg(Brewtarget::displayAmount(recObs->getEfficiency_pct(),0,0));
+   // second row:  boil time and efficiency.  
+   body += QString("<tr><td class=\"left\">%1</td><td class=\"value\">%2</td><td class=\"right\">%3</td><td class=\"value\">%4</td></tr>")
+            .arg(tr("Boil Time"))
+            .arg(Brewtarget::displayAmount(recObs->getEquipment()->getBoilTime_min(),Units::minutes))
+            .arg(tr("Efficiency"))
+            .arg(Brewtarget::displayAmount(recObs->getEfficiency_pct(),0,0));
 
    // third row: pre-Boil Volume and Preboil Gravity
    body += QString("<tr><td class=\"left\">%1</td><td class=\"value\">%2</td><td class=\"right\">%3</td><td class=\"value\">%4</td></tr>")
-		   .arg(tr("Boil Volume"))
-           .arg(Brewtarget::displayAmount(recObs->getBoilSize_l(),Units::liters,2))
-           .arg(tr("Preboil Gravity"))
-           .arg(Brewtarget::displayOG(recObs->getBoilGrav()));
+            .arg(tr("Boil Volume"))
+            .arg(Brewtarget::displayAmount(recObs->estimateBoilVolume_l(),Units::liters,2))
+            .arg(tr("Preboil Gravity"))
+            .arg(Brewtarget::displayOG(recObs->getBoilGrav()));
 
    // fourth row: Final volume and starting gravity
    body += QString("<tr><td class=\"left\">%1</td><td class=\"value\">%2</td><td class=\"right\">%3</td><td class=\"value\">%4</td></tr>")
-		   .arg(tr("Final Volume"))
-           .arg(Brewtarget::displayAmount(recObs->getBatchSize_l(), Units::liters,2))
-           .arg(tr("Starting Gravity"))
-           .arg(Brewtarget::displayOG(recObs->getOg(), true));
+            .arg(tr("Final Volume"))
+            .arg(Brewtarget::displayAmount(recObs->estimateFinalVolume_l(), Units::liters,2))
+            .arg(tr("Starting Gravity"))
+            .arg(Brewtarget::displayOG(recObs->getOg(), true));
 
    // fifth row: IBU and Final gravity
    body += QString("<tr><td class=\"left\">%1</td><td class=\"value\">%2</td><td class=\"right\">%3</td><td class=\"value\">%4</tr>")
-		   .arg(tr("IBU"))
-           .arg( recObs->getIBU(),0,'f',1)
-           .arg(tr("Final Gravity"))
-           .arg(Brewtarget::displayFG(recObs->getFg(), recObs->getOg(), true));
+            .arg(tr("IBU"))
+            .arg( recObs->getIBU(),0,'f',1)
+            .arg(tr("Final Gravity"))
+            .arg(Brewtarget::displayFG(recObs->getFg(), recObs->getOg(), true));
 
    // sixth row: ABV and estimate calories
    body += QString("<tr><td class=\"left\">%1</td><td class=\"value\">%2%</td><td class=\"right\">%3</td><td class=\"value\">%4</tr>")
-		   .arg(tr("ABV"))
-		   .arg( recObs->getABV_pct(), 0, 'f', 1)
-		   .arg(tr("Estimated calories(per 12 oz)"))
-		   .arg( recObs->estimateCalories(), 0, 'f', 0);
+            .arg(tr("ABV"))
+            .arg( recObs->getABV_pct(), 0, 'f', 1)
+            .arg(tr("Estimated calories(per 12 oz)"))
+            .arg( recObs->estimateCalories(), 0, 'f', 0);
    body += "</table>";
 
    return header + body;
@@ -205,9 +213,9 @@ QString BrewDayScrollWidget::buildInstructionTable()
    middle += QString("<h2>%1</h2>").arg(tr("Instructions"));
    middle += QString("<table id=\"steps\">");
    middle += QString("<tr><th class=\"check\">%1</th><th class=\"time\">%2</th><th class=\"step\">%3</th></tr>")
-		   .arg(tr("Completed"))
-		   .arg(tr("Time"))
-		   .arg(tr("Step"));
+         .arg(tr("Completed"))
+         .arg(tr("Time"))
+         .arg(tr("Step"));
 
    size = recObs->getNumInstructions();
    for( i = 0; i < size; ++i )
@@ -231,9 +239,9 @@ QString BrewDayScrollWidget::buildInstructionTable()
       }
       else if ( ins->getName() == "Heat water")
       {
-    	  int mashSize = recObs->getMash()->getNumMashSteps();
-    	  Instruction* temp = recObs->getMashWater(mashSize);
-    	  reagents = temp->getReagents();
+         int mashSize = recObs->getMash()->getNumMashSteps();
+         Instruction* temp = recObs->getMashWater(mashSize);
+         reagents = temp->getReagents();
 
       }
 
@@ -249,7 +257,7 @@ QString BrewDayScrollWidget::buildInstructionTable()
       }
       else
       {
-    	  tmp = reagents[0];
+         tmp = reagents[0];
       }
 
       QString altTag = i % 2 ? "alt" : "norm";
@@ -271,15 +279,15 @@ QString BrewDayScrollWidget::buildFooterTable()
 
    bottom = QString("<table id=\"notes\">");
    bottom += QString("<tr><td class=\"left\">%1:</td><td class=\"value\"></td><td class=\"right\">%2:</td><td class=\"value\"></td></tr>")
-		   .arg(tr("Actual PreBoil Volume"))
-		   .arg(tr("Actual PreBoil Gravity"));
+         .arg(tr("Actual PreBoil Volume"))
+         .arg(tr("Actual PreBoil Gravity"));
 
    bottom += QString("<tr><td class=\"left\">%1:</td><td class=\"value\"></td><td class=\"right\">%2:</td><td class=\"value\"></td></tr>")
-		   .arg(tr("PostBoil Volume"))
-		   .arg(tr("PostBoil Gravity"));
+         .arg(tr("PostBoil Volume"))
+         .arg(tr("PostBoil Gravity"));
 
    bottom += QString("<tr><td class=\"left\">%1:</td><td class=\"value\"></tr>")
-		   .arg(tr("Volume into fermenter"));
+         .arg(tr("Volume into fermenter"));
    bottom += "</table>";
 
    return bottom;
@@ -292,7 +300,8 @@ bool BrewDayScrollWidget::loadComplete(bool ok)
    return ok;
 }
 
-void BrewDayScrollWidget::print(QPrinter *mainPrinter, QPrintDialog* dialog)
+void BrewDayScrollWidget::print(QPrinter *mainPrinter, QPrintDialog* dialog,
+      int action, QFile* outFile)
 {
    QString pDoc;
 //   QPrintDialog *dialog = new QPrintDialog(printer, this);
@@ -300,46 +309,38 @@ void BrewDayScrollWidget::print(QPrinter *mainPrinter, QPrintDialog* dialog)
    if( recObs == 0 )
       return;
 
-   printer = mainPrinter;
-
    /* Connect the webview's signal */
-   connect( doc, SIGNAL(loadFinished(bool)), this, SLOT(loadComplete(bool)) );
+   if ( action == PRINT )
+   {
+      printer = mainPrinter;
+      connect( doc, SIGNAL(loadFinished(bool)), this, SLOT(loadComplete(bool)) );
 
-   dialog->setWindowTitle(tr("Print Document"));
-   if (dialog->exec() != QDialog::Accepted)
-      return;
+      dialog->setWindowTitle(tr("Print Document"));
+      if (dialog->exec() != QDialog::Accepted)
+         return;
+   }
 
-   // Start building the document to be printed.  I think.
-   pDoc = buildTitleTable();
+   // Start building the document to be printed.  The HTML doesn't work with
+   // the image since it is a compiled resource
+   pDoc = buildTitleTable( action != HTML );
    pDoc += buildInstructionTable();
    pDoc += buildFooterTable();
 
    pDoc += tr("<h2>Notes</h2>");
    if ( recObs->getNotes() != "" )
-	   pDoc += QString("%1").arg(recObs->getNotes());
+      pDoc += QString("%1").arg(recObs->getNotes());
 
    pDoc += "</body></html>";
 
    doc->setHtml(pDoc);
-}
-
-void BrewDayScrollWidget::printPreview()
-{
-   QString pDoc;
-
-   if( recObs == 0 )
-      return;
-
-   // Start building the document to be printed.  I think.
-   pDoc = buildTitleTable();
-   pDoc += buildInstructionTable();
-   pDoc += buildFooterTable();
-
-   pDoc += tr("<h2>Notes</h2>");
-   pDoc += "</body></html>";
-
-   doc->setHtml(pDoc);
-   doc->show();
+   if ( action == PREVIEW )
+      doc->show();
+   else if ( action == HTML )
+   {
+      QTextStream out(outFile);
+      out << pDoc;
+      outFile->close();
+   }
 }
 
 void BrewDayScrollWidget::setRecipe(Recipe* rec)
@@ -379,51 +380,13 @@ void BrewDayScrollWidget::notify(Observable* notifier, QVariant info)
 void BrewDayScrollWidget::clear()
 {
    listWidget->clear();
-
-   /*
-   while( stackedWidget->count() > 0 )
-   {
-      InstructionWidget* iw = (InstructionWidget*)stackedWidget->widget(0);
-      stackedWidget->removeWidget(iw);
-      delete iw;
-   }
-
-   stackedWidget->setCurrentIndex(0);
-   */
 }
 
 void BrewDayScrollWidget::showChanges()
 {
    clear();
    if( recObs == 0 )
-   {
-      //clear();
       return;
-   }
-
-   /*
-   int i, size;
-   InstructionWidget* iw;
-   size = recObs->getNumInstructions();
-
-   for( i = 0; i < size; ++i )
-   {
-      if(stackedWidget->widget(i) == 0)
-      {
-         iw = new InstructionWidget(stackedWidget);
-         stackedWidget->addWidget(iw);
-      }
-      else
-      {
-         iw = (InstructionWidget*)stackedWidget->widget(i);
-         iw->setVisible(true);
-      }
-
-      iw->setInstruction(recObs->getInstruction(i));
-   }
-
-   stackedWidget->update(); // Whatever, I give up.
-   */
 
    repopulateListWidget();
 }
