@@ -17,7 +17,7 @@
  */
 
 #ifndef _EQUIPMENTEDITOR_H
-#define	_EQUIPMENTEDITOR_H
+#define   _EQUIPMENTEDITOR_H
 
 class EquipmentEditor;
 
@@ -32,7 +32,9 @@ class EquipmentEditor : public QDialog, public Ui::equipmentEditor, public Obser
 
 public:
    EquipmentEditor( QWidget *parent=0 );
+   virtual ~EquipmentEditor() {}
    void setEquipment( Equipment* e );
+   void resetEquipment();
 
 public slots:
    void save();
@@ -43,13 +45,20 @@ public slots:
    void resetAbsorption();
 
    void equipmentSelected( const QString& text );
+   void updateRecord();
+   void updateCheckboxRecord(int state);
+   void changedText();
 
 private:
    Equipment* obsEquip;
+   Equipment* copyEquip;
+
+   bool changeText;
 
    virtual void notify(Observable* notifier, QVariant info = QVariant()); // Inherited from Observer
+   bool eventFilter(QObject *object, QEvent* event);
    void showChanges();
 };
 
-#endif	/* _EQUIPMENTEDITOR_H */
+#endif   /* _EQUIPMENTEDITOR_H */
 
