@@ -17,7 +17,7 @@
  */
 
 #ifndef _HOPDIALOG_H
-#define	_HOPDIALOG_H
+#define   _HOPDIALOG_H
 
 class HopDialog;
 
@@ -36,11 +36,12 @@ class HopDialog : public QDialog, public Ui::hopDialog, public Observer
 
 public:
    HopDialog(MainWindow* parent);
+   virtual ~HopDialog() {}
    void startObservingDB();
    virtual void notify(Observable *notifier, QVariant info = QVariant()); // From Observer
 
 public slots:
-   void addHop();
+   void addHop(const QModelIndex& = QModelIndex());
    void removeHop();
    void editSelected();
    void newHop();
@@ -55,5 +56,5 @@ private:
 };
 
 
-#endif	/* _HOPDIALOG_H */
+#endif   /* _HOPDIALOG_H */
 
