@@ -75,6 +75,17 @@ void MashDesigner::setRecipe(Recipe* rec)
 
 void MashDesigner::show()
 {
+   // No point to run unless we have fermentables.
+   if( recObs && recObs->getNumFermentables() == 0 )
+   {
+      QMessageBox::information(
+			   this,
+         tr("No Fermentables"),
+         tr("Your recipe must have fermentables to design a mash.")
+      );
+      return;
+   }
+
    setVisible(nextStep(0));
 }
 
@@ -147,7 +158,12 @@ void MashDesigner::saveStep()
 
 double MashDesigner::maxTemp_c()
 {
-   return 100;
+   if ( recObs && recObs->getEquipment())
+   {
+      return recObs->getEquipment()->getBoilingPoint_c();
+   }
+   else
+      return 100;
 }
 
 double MashDesigner::minTemp_c()
@@ -263,24 +279,24 @@ bool MashDesigner::initializeMash()
 
    mash->setTunSpecificHeat_calGC( equip->getTunSpecificHeat_calGC() );
    mash->setTunWeight_kg( equip->getTunWeight_kg() );
-	
-	bool ok;
-	QString dialogText = QInputDialog::getText(
-													 this,
-													 tr("Tun Temp"),
-													 tr("Enter the temperature of the tun before your first infusion."),
-													 QLineEdit::Normal, //default,
-													 QString::null,
-													 &ok
-													//don't need the widget pointer - default is parent
-															 );
-	
-	//if user hits cancel, cancel out of the dialog and quit the mashDesigner
-	//edited jazzbeerman (dcavanagh) 8/20/10
-	if(!ok)
-		return false;
-	
-	//otherwise continue - get the text and keep going
+   
+   bool ok;
+   QString dialogText = QInputDialog::getText(
+                                        this,
+                                        tr("Tun Temp"),
+                                        tr("Enter the temperature of the tun before your first infusion."),
+                                        QLineEdit::Normal, //default,
+                                        QString::null,
+                                        &ok
+                                       //don't need the widget pointer - default is parent
+                                              );
+   
+   //if user hits cancel, cancel out of the dialog and quit the mashDesigner
+   //edited jazzbeerman (dcavanagh) 8/20/10
+   if(!ok)
+      return false;
+   
+   //otherwise continue - get the text and keep going
    mash->setTunTemp_c( Brewtarget::tempQStringToSI( dialogText ) );
 
    mash->removeAllMashSteps();
@@ -322,6 +338,10 @@ void MashDesigner::updateFullness()
    else
       vol_l = grainVolume_l() + getSelectedAmount_l();
    double ratio = vol_l / equip->getTunVolume_l();
+   if( ratio < 0 )
+     ratio = 0;
+   if( ratio > 1 )
+     ratio = 1;
 
    progressBar_fullness->setValue(ratio*progressBar_fullness->maximum());
    label_mashVol->setText(Brewtarget::displayAmount(vol_l, Units::liters));
@@ -335,6 +355,10 @@ void MashDesigner::updateCollectedWort()
 
    double wort_l = recObs->estimateWortFromMash_l();
    double ratio = wort_l / recObs->getBoilSize_l();
+   if( ratio < 0 )
+     ratio = 0;
+   if( ratio > 1 )
+     ratio = 1;
 
    label_wort->setText(Brewtarget::displayAmount(wort_l, Units::liters));
    progressBar_wort->setValue( ratio * progressBar_wort->maximum() );
