@@ -17,7 +17,7 @@
  */
 
 #ifndef _MISCDIALOG_H
-#define	_MISCDIALOG_H
+#define   _MISCDIALOG_H
 
 class MiscDialog;
 
@@ -36,11 +36,12 @@ class MiscDialog : public QDialog, public Ui::miscDialog, public Observer
 
 public:
    MiscDialog(MainWindow* parent);
+   virtual ~MiscDialog() {}
    void startObservingDB();
    virtual void notify(Observable *notifier, QVariant info = QVariant()); // From Observer
 
 public slots:
-   void addMisc();
+   void addMisc(const QModelIndex& = QModelIndex());
    void removeMisc();
    void editSelected();
    void newMisc();
@@ -54,5 +55,5 @@ private:
    void populateTable();
 };
 
-#endif	/* _MISCDIALOG_H */
+#endif   /* _MISCDIALOG_H */
 
