@@ -89,8 +89,8 @@ QString RecipeFormatter::getTextFormat()
    
    ret += getTextSeparator();
    
-   ret += QObject::tr("Batch Size: %1\n").arg(Brewtarget::displayAmount(rec->getBatchSize_l(), Units::liters));
-   ret += QObject::tr("Boil Size: %1\n").arg(Brewtarget::displayAmount(rec->getBoilSize_l(), Units::liters));
+   ret += QObject::tr("Batch Size: %1\n").arg(Brewtarget::displayAmount(rec->estimateFinalVolume_l(), Units::liters));
+   ret += QObject::tr("Boil Size: %1\n").arg(Brewtarget::displayAmount(rec->estimateBoilVolume_l(), Units::liters));
    ret += QObject::tr("Boil Time: %1\n").arg( (rec->getEquipment() == 0)?
                                               Brewtarget::displayAmount(0, Units::minutes)
                                             : Brewtarget::displayAmount( (rec->getEquipment())->getBoilTime_min(), Units::minutes));
@@ -376,8 +376,9 @@ QString RecipeFormatter::getHTMLFormat()
    pDoc += buildMashTable();
    pDoc += buildNotes();
    pDoc += buildInstructionTable();
+   pDoc += buildBrewNotes();
 
-   pDoc += "</body></html>";
+   pDoc += "</div></body></html>";
 
    return pDoc;
 }
@@ -448,7 +449,7 @@ QString RecipeFormatter::buildTitleTable()
    Style* style = 0;
 
    if ( rec == 0 )
-	   return "";
+      return "";
 
    style = rec->getStyle();
 
@@ -469,21 +470,25 @@ QString RecipeFormatter::buildTitleTable()
 
    body += QString("<tr><td class=\"label\">%1</td><td class=\"value\">%2</td></tr>")
            .arg(tr("Brewer"))
-		   .arg(rec->getBrewer());
-   body += QString("<tr><td class=\"label\">%1</td><td class=\"value\">%2</td></tr>")
+         .arg(rec->getBrewer());
+   body += QString("<tr><td class=\"label\">%1</td><td class=\"value \">%2</td></tr>")
            .arg(tr("Date"))
-		   .arg(rec->getDate());
+         .arg(rec->getDate());
    body += "</table>";
 
    // Build the top table
-   // Build the first row: Batch Size and Boil Size
+   // Build the first row: Batch Size and Boil Size.
+   // NOTE: using getBatchSize_l() and/or getBoilSize_l() only gives the
+   // *target* batch and boil size.  I think we want the actual (aka,
+   // estimated) sizes
+
    body += "<table id=\"title\">";
    body += QString("<tr><td class=\"left\">%1</td><td class=\"value\">%2</td>")
            .arg(tr("Batch Size"))
-           .arg(Brewtarget::displayAmount(rec->getBatchSize_l(), Units::liters));
+           .arg(Brewtarget::displayAmount(rec->estimateFinalVolume_l(), Units::liters));
    body += QString("<td class=\"right\">%1</td><td class=\"value\">%2</td></tr>")
            .arg(tr("Boil Size"))
-           .arg(Brewtarget::displayAmount(rec->getBoilSize_l(), Units::liters));
+           .arg(Brewtarget::displayAmount(rec->estimateBoilVolume_l(), Units::liters));
    // Second row: Boil Time and Efficiency
    body += QString("<tr><td class=\"left\">%1</td><td class=\"value\">%2</td>")
            .arg(tr("Boil Time"))
@@ -504,15 +509,15 @@ QString RecipeFormatter::buildTitleTable()
 
    // Fourth row: ABV and Bitterness.  We need to set the bitterness string up first
    bitterness = QString("%1 IBU (%2)")
-		   .arg( rec->getIBU(), 0, 'f', 1);
+         .arg( rec->getIBU(), 0, 'f', 1);
    switch ( Brewtarget::ibuFormula )
    {
-	   case Brewtarget::TINSETH:
-		   bitterness = bitterness.arg("Tinseth");
-		   break;
-	   case Brewtarget::RAGER:
-		   bitterness = bitterness.arg("Rager");
-		   break;
+      case Brewtarget::TINSETH:
+         bitterness = bitterness.arg("Tinseth");
+         break;
+      case Brewtarget::RAGER:
+         bitterness = bitterness.arg("Rager");
+         break;
        default:
            bitterness = bitterness.arg(tr("Unknown"));
    }
@@ -523,18 +528,18 @@ QString RecipeFormatter::buildTitleTable()
            .arg(tr("Bitterness"))
            .arg(bitterness);
    // Fifth row: Color and calories.  Set up the color string first
-   color = tr("%1 SRM (%2)").arg(rec->getColor_srm(), 0, 'f', 0);
+   color = QString("%1 (%2)").arg(Brewtarget::displayColor(rec->getColor_srm(),true));
    switch( Brewtarget::colorFormula )
    {
-	   case Brewtarget::MOREY:
-		   color = color.arg("Morey");
-		   break;
-	   case Brewtarget::DANIEL:
-		   color = color.arg("Daniels");
-		   break;
-	   case Brewtarget::MOSHER:
-		   color = color.arg("Mosher");
-		   break;
+      case Brewtarget::MOREY:
+         color = color.arg("Morey");
+         break;
+      case Brewtarget::DANIEL:
+         color = color.arg("Daniels");
+         break;
+      case Brewtarget::MOSHER:
+         color = color.arg("Mosher");
+         break;
    }
    body += QString("<tr><td class=\"left\">%1</td><td class=\"value\">%2</td>")
            .arg(tr("Color"))
@@ -551,139 +556,139 @@ QString RecipeFormatter::buildTitleTable()
 
 QString RecipeFormatter::buildFermentableTable()
 {
-	QString ftable;
-	if ( rec == 0 || rec->getNumFermentables() < 1 )
-		return "";
+   QString ftable;
+   if ( rec == 0 || rec->getNumFermentables() < 1 )
+      return "";
 
-	ftable = QString("<h3>%1</h3>").arg(tr("Fermentables"));
+   ftable = QString("<h3>%1</h3>").arg(tr("Fermentables"));
    ftable += QString("<table id=\"fermentables\">");
-	ftable += QString("<caption>%1 %2</caption>")
+   ftable += QString("<caption>%1 %2</caption>")
                   .arg(tr("Total grain:"))
                   .arg(Brewtarget::displayAmount(rec->getGrains_kg(), Units::kilograms));
-	// Set up the header row.
-	ftable += QString("<tr><th>%1</th><th>%2</th><th>%3</th><th>%4</th><th>%5</th><th>%6</th><th>%7</th></tr>")
-			.arg(tr("Name"))
-			.arg(tr("Type"))
-			.arg(tr("Amount"))
-			.arg(tr("Mashed"))
-			.arg(tr("Late"))
-			.arg(tr("Yield"))
-			.arg(tr("Color"));
-	// Now add a row for each fermentable
-	for(unsigned int i=0; i < rec->getNumFermentables(); ++i)
-	{
-		Fermentable* ferm = rec->getFermentable(i);
-		ftable += "<tr>";
-		ftable += QString("<td>%1</td><td>%2</td><td>%3</td><td>%4</td><td>%5</td><td>%6%</td><td>%7 L</td>")
+   // Set up the header row.
+   ftable += QString("<tr><th>%1</th><th>%2</th><th>%3</th><th>%4</th><th>%5</th><th>%6</th><th>%7</th></tr>")
+         .arg(tr("Name"))
+         .arg(tr("Type"))
+         .arg(tr("Amount"))
+         .arg(tr("Mashed"))
+         .arg(tr("Late"))
+         .arg(tr("Yield"))
+         .arg(tr("Color"));
+   // Now add a row for each fermentable
+   for(unsigned int i=0; i < rec->getNumFermentables(); ++i)
+   {
+      Fermentable* ferm = rec->getFermentable(i);
+      ftable += "<tr>";
+      ftable += QString("<td>%1</td><td>%2</td><td>%3</td><td>%4</td><td>%5</td><td>%6%</td><td>%7</td>")
             .arg( ferm->getName())
-				.arg( ferm->getTypeStringTr())
-				.arg( Brewtarget::displayAmount(ferm->getAmount_kg(), Units::kilograms))
-				.arg( ferm->getIsMashed() ? tr("Yes") : tr("No") )
-				.arg( ferm->getAddAfterBoil() ? tr("Yes") : tr("No"))
-				.arg( ferm->getYield_pct(), 0, 'f', 0)
-				.arg( ferm->getColor_srm(), 0, 'f', 0);
-		ftable += "</tr>";
-	}
-	ftable += "</table>";
-	return ftable;
+            .arg( ferm->getTypeStringTr())
+            .arg( Brewtarget::displayAmount(ferm->getAmount_kg(), Units::kilograms))
+            .arg( ferm->getIsMashed() ? tr("Yes") : tr("No") )
+            .arg( ferm->getAddAfterBoil() ? tr("Yes") : tr("No"))
+            .arg( ferm->getYield_pct(), 0, 'f', 0)
+            .arg( Brewtarget::displayColor(ferm->getColor_srm(),true));
+      ftable += "</tr>";
+   }
+   ftable += "</table>";
+   return ftable;
 }
 
 QString RecipeFormatter::buildHopsTable()
 {
-	QString hTable;
+   QString hTable;
 
-	if ( rec == 0 || rec->getNumHops() < 1 )
-		return "";
+   if ( rec == 0 || rec->getNumHops() < 1 )
+      return "";
 
-	hTable = QString("<h3>%1</h3>").arg(tr("Hops"));
+   hTable = QString("<h3>%1</h3>").arg(tr("Hops"));
    hTable += QString("<table id=\"hops\">");
-	// Set up the header row.
-	hTable += QString("<tr><th>%1</th><th>%2</th><th>%3</th><th>%4</th><th>%5</th><th>%6</th><th>%7</th></tr>")
-			.arg(tr("Name"))
-			.arg(tr("Alpha"))
-			.arg(tr("Amount"))
-			.arg(tr("Use"))
-			.arg(tr("Time"))
-			.arg(tr("Form"))
-			.arg(tr("IBU"));
-	for( unsigned int i = 0; i < rec->getNumHops(); ++i)
-	{
-		Hop *hop = rec->getHop(i);
-      hTable += QString("<td>%1</td><td>%2%</td><td>%3</td><td>%4</td><td>%5</td><td>%6</td><td>%7</td>")
+   // Set up the header row.
+   hTable += QString("<tr><th>%1</th><th>%2</th><th>%3</th><th>%4</th><th>%5</th><th>%6</th><th>%7</th></tr>")
+         .arg(tr("Name"))
+         .arg(tr("Alpha"))
+         .arg(tr("Amount"))
+         .arg(tr("Use"))
+         .arg(tr("Time"))
+         .arg(tr("Form"))
+         .arg(tr("IBU"));
+   for( unsigned int i = 0; i < rec->getNumHops(); ++i)
+   {
+      Hop *hop = rec->getHop(i);
+      hTable += QString("<tr><td>%1</td><td>%2%</td><td>%3</td><td>%4</td><td>%5</td><td>%6</td><td>%7</td></tr>")
             .arg( hop->getName())
-				.arg( hop->getAlpha_pct(), 0, 'f', 0)
-				.arg( Brewtarget::displayAmount(hop->getAmount_kg(), Units::kilograms))
-				.arg( hop->getUseStringTr())
-				.arg( Brewtarget::displayAmount(hop->getTime_min(), Units::minutes) )
-				.arg( hop->getFormStringTr())
-				.arg( rec->getIBUFromHop(i), 0, 'f', 1);
-		hTable += "</tr>";
-	}
-	hTable += "</table>";
-	return hTable;
+            .arg( hop->getAlpha_pct(), 0, 'g', 3)
+            .arg( Brewtarget::displayAmount(hop->getAmount_kg(), Units::kilograms))
+            .arg( hop->getUseStringTr())
+            .arg( Brewtarget::displayAmount(hop->getTime_min(), Units::minutes) )
+            .arg( hop->getFormStringTr())
+            .arg( rec->getIBUFromHop(i), 0, 'f', 1);
+      hTable += "</tr>";
+   }
+   hTable += "</table>";
+   return hTable;
 }
 
 QString RecipeFormatter::buildMiscTable()
 {
-	QString mtable;
+   QString mtable;
 
-	if ( rec == 0 || rec->getNumMiscs() < 1 )
-		return "";
+   if ( rec == 0 || rec->getNumMiscs() < 1 )
+      return "";
 
-	mtable = QString("<h3>%1</h3>").arg(tr("Misc"));
+   mtable = QString("<h3>%1</h3>").arg(tr("Misc"));
    mtable += QString("<table id=\"misc\">");
-	// Set up the header row.
-	mtable += QString("<tr><th>%1</th><th>%2</th><th>%3</th><th>%4</th><th>%5</th></tr>")
-			.arg(tr("Name"))
-			.arg(tr("Type"))
-			.arg(tr("Use"))
-			.arg(tr("Amount"))
-			.arg(tr("Time"));
-	for( unsigned int i = 0; i < rec->getNumMiscs(); ++i)
-	{
-		Misc *misc = rec->getMisc(i);
-		mtable += QString("<td>%1</td><td>%2</td><td>%3</td><td>%4</td><td>%5</td>")
+   // Set up the header row.
+   mtable += QString("<tr><th>%1</th><th>%2</th><th>%3</th><th>%4</th><th>%5</th></tr>")
+         .arg(tr("Name"))
+         .arg(tr("Type"))
+         .arg(tr("Use"))
+         .arg(tr("Amount"))
+         .arg(tr("Time"));
+   for( unsigned int i = 0; i < rec->getNumMiscs(); ++i)
+   {
+      Misc *misc = rec->getMisc(i);
+      mtable += QString("<td>%1</td><td>%2</td><td>%3</td><td>%4</td><td>%5</td>")
             .arg( misc->getName())
-				.arg( misc->getTypeStringTr())
-				.arg( misc->getUseStringTr())
-				.arg( Brewtarget::displayAmount(misc->getAmount(), misc->getAmountIsWeight() ? (Unit*)Units::kilograms : (Unit*)Units::liters))
-				.arg( Brewtarget::displayAmount(misc->getTime(), Units::minutes) );
-		mtable += "</tr>";
-	}
-	mtable += "</table>";
-	return mtable;
+            .arg( misc->getTypeStringTr())
+            .arg( misc->getUseStringTr())
+            .arg( Brewtarget::displayAmount(misc->getAmount(), misc->getAmountIsWeight() ? (Unit*)Units::kilograms : (Unit*)Units::liters))
+            .arg( Brewtarget::displayAmount(misc->getTime(), Units::minutes) );
+      mtable += "</tr>";
+   }
+   mtable += "</table>";
+   return mtable;
 
 }
 
 QString RecipeFormatter::buildYeastTable()
 {
-	QString ytable;
+   QString ytable;
 
-	if ( rec == 0 || rec->getNumYeasts() < 1 )
-		return "";
+   if ( rec == 0 || rec->getNumYeasts() < 1 )
+      return "";
 
-	ytable = QString("<h3>%1</h3>").arg(tr("Yeast"));
+   ytable = QString("<h3>%1</h3>").arg(tr("Yeast"));
    ytable += QString("<table id=\"yeast\">");
-	// Set up the header row.
-	ytable += QString("<tr><th>%1</th><th>%2</th><th>%3</th><th>%4</th><th>%5</th></tr>")
-			.arg(tr("Name"))
-			.arg(tr("Type"))
-			.arg(tr("Form"))
-			.arg(tr("Amount"))
-			.arg(tr("Stage"));
-	for( unsigned int i = 0; i < rec->getNumYeasts(); ++i)
-	{
-		Yeast *y = rec->getYeast(i);
-		ytable += QString("<td>%1</td><td>%2</td><td>%3</td><td>%4</td><td>%5</td>")
+   // Set up the header row.
+   ytable += QString("<tr><th>%1</th><th>%2</th><th>%3</th><th>%4</th><th>%5</th></tr>")
+         .arg(tr("Name"))
+         .arg(tr("Type"))
+         .arg(tr("Form"))
+         .arg(tr("Amount"))
+         .arg(tr("Stage"));
+   for( unsigned int i = 0; i < rec->getNumYeasts(); ++i)
+   {
+      Yeast *y = rec->getYeast(i);
+      ytable += QString("<tr><td>%1</td><td>%2</td><td>%3</td><td>%4</td><td>%5</td></tr>")
             .arg( y->getName())
-				.arg( y->getTypeStringTr())
-				.arg( y->getFormStringTr())
-				.arg( Brewtarget::displayAmount( y->getAmount(), y->getAmountIsWeight() ? (Unit*)Units::kilograms : (Unit*)Units::liters ) )
-				.arg( y->getAddToSecondary() ? tr("Secondary") : tr("Primary"));
-		ytable += "</tr>";
-	}
-	ytable += "</table>";
-	return ytable;
+            .arg( y->getTypeStringTr())
+            .arg( y->getFormStringTr())
+            .arg( Brewtarget::displayAmount( y->getAmount(), y->getAmountIsWeight() ? (Unit*)Units::kilograms : (Unit*)Units::liters ) )
+            .arg( y->getAddToSecondary() ? tr("Secondary") : tr("Primary"));
+      ytable += "</tr>";
+   }
+   ytable += "</table>";
+   return ytable;
 }
 
 QString RecipeFormatter::buildMashTable()
@@ -743,15 +748,15 @@ QString RecipeFormatter::buildMashTable()
 
 QString RecipeFormatter::buildNotes()
 {
-	QString notes;
+   QString notes;
 
-	if ( rec == 0 || rec->getNotes() == "" )
-		return "";
+   if ( rec == 0 || rec->getNotes() == "" )
+      return "";
 
-	notes = QString("<h3>%1</h3>").arg(tr("Notes"));
-	notes += QString("%1").arg( rec->getNotes());
+   notes = QString("<h3>%1</h3>").arg(tr("Notes"));
+   notes += QString("%1").arg( rec->getNotes());
 
-	return notes;
+   return notes;
 }
 
 QString RecipeFormatter::buildInstructionTable()
@@ -759,22 +764,96 @@ QString RecipeFormatter::buildInstructionTable()
    QString itable;
 
    if ( rec == 0 || rec->getNumInstructions() < 1 )
-	   return "";
+      return "";
 
    itable = QString("<h3>%1</h3>").arg(tr("Instructions"));
    itable += "<ol id=\"instruction\">";
 
    for(int i = 0; i < rec->getNumInstructions(); ++i )
    {
-	   Instruction* ins = rec->getInstruction(i);
-	   itable += QString("<li>%1</li>").arg( ins->getDirections());
+      Instruction* ins = rec->getInstruction(i);
+      itable += QString("<li>%1</li>").arg( ins->getDirections());
    }
 
-   itable += "</table>";
+   itable += "</ol>";
 
    return itable;
 }
 
+QString RecipeFormatter::buildBrewNotes()
+{
+   QString bnTable = "";
+
+   if ( rec == 0 || rec->getNumBrewNotes() < 1 )
+      return bnTable;
+
+   for( unsigned int i = 0; i < rec->getNumBrewNotes(); ++i )
+   {
+      BrewNote* note = rec->getBrewNote(i);
+
+      bnTable += QString("<h2>%1 %2</h2>").arg(tr("Brew Date")).arg(note->getBrewDate_short());
+      
+      // PREBOIL, done two-by-two
+      bnTable += "<table id=\"brewnote\">";
+      bnTable += QString("<caption>%1</caption>").arg(tr("Preboil"));
+      bnTable += QString("<tr><td class=\"left\">%1</td><td class=\"value\">%2</td><td class=\"right\">%3</td><td class=\"value\">%4</td></tr>")
+                 .arg(tr("SG"))
+                 .arg(Brewtarget::displayOG(note->getSG(),true))
+                 .arg(tr("Volume into BK"))
+                 .arg(Brewtarget::displayAmount(note->getVolumeIntoBK_l(), Units::liters));
+
+      bnTable += QString("<tr><td class=\"left\">%1</td><td class=\"value\">%2</td><td class=\"right\">%3</td><td class=\"value\">%4</td></tr>")
+                 .arg(tr("Strike Temp"))
+                 .arg(Brewtarget::displayAmount(note->getStrikeTemp_c(), Units::celsius))
+                 .arg(tr("Final Temp"))
+                 .arg(Brewtarget::displayAmount(note->getMashFinTemp_c(), Units::celsius));
+
+      bnTable += QString("<tr><td class=\"left\">%1</td><td class=\"value\">%2%</td><td class=\"right\">%3</td><td class=\"value\">%4</td></tr>")
+                 .arg(tr("Eff into BK"))
+                 .arg(note->calculateEffIntoBK_pct(), 0, 'f', 2)
+                 .arg(tr("Projected OG"))
+                 .arg(Brewtarget::displayOG(note->calculateOG(), true));
+      bnTable += "</table>";
+
+      // POSTBOIL
+      bnTable += "<table id=\"brewnote\">";
+      bnTable += QString("<caption>%1</caption>").arg(tr("Postboil"));
+      bnTable += QString("<tr><td class=\"left\">%1</td><td class=\"value\">%2</td><td class=\"right\">%3</td><td class=\"value\">%4</td></tr>")
+                 .arg(tr("OG"))
+                 .arg(Brewtarget::displayOG(note->getOG(),true))
+                 .arg(tr("Postboil Volume"))
+                 .arg(Brewtarget::displayAmount(note->getPostBoilVolume_l(), Units::liters));
+      bnTable += QString("<tr><td class=\"left\">%1</td><td class=\"value\">%2</td><td class=\"right\">%3</td><td class=\"value\">%4</td></tr>")
+                 .arg(tr("Volume Into Fermenter"))
+                 .arg(Brewtarget::displayAmount(note->getVolumeIntoFerm_l(), Units::liters))
+                 .arg(tr("Brewhouse Eff"))
+                 .arg(note->calculateBrewHouseEff_pct(), 0, 'f', 2);
+      bnTable += QString("<tr><td class=\"left\">%1</td><td class=\"value\">%2%</td></tr>")
+                 .arg(tr("Projected ABV"))
+                 .arg(note->getProjABV_pct(), 0, 'f', 2);
+      bnTable += "</table>";
+
+
+      // POSTFERMENT
+      bnTable += "<table id=\"brewnote\">";
+      bnTable += QString("<caption>%1</caption>").arg(tr("Postferment"));
+      bnTable += QString("<tr><td class=\"left\">%1</td><td class=\"value\">%2</td><td class=\"right\">%3</td><td class=\"value\">%4</td></tr>")
+                 .arg(tr("FG"))
+                 .arg(Brewtarget::displayOG(note->getFG(),true))
+                 .arg(tr("Volume"))
+                 .arg(Brewtarget::displayAmount(note->getFinalVolume_l(), Units::liters));
+      bnTable += QString("<tr><td class=\"left\">%1</td><td class=\"value\">%2</td><td class=\"right\">%3</td><td class=\"value\">%4</td></tr>")
+                 .arg(tr("Date"))
+                 .arg(note->getFermentDate_short())
+                 .arg(tr("ABV"))
+                 .arg(note->actualABV_pct(), 0, 'f', 2);
+      bnTable += "</table>";
+
+   }
+
+   return bnTable;
+}
+      
 bool RecipeFormatter::loadComplete(bool ok)
 {
    doc->print(printer);
@@ -782,27 +861,32 @@ bool RecipeFormatter::loadComplete(bool ok)
    return ok;
 }
 
-void RecipeFormatter::print(QPrinter* mainPrinter, QPrintDialog *dialog)
-{
-   printer = mainPrinter;
-
-   dialog->setWindowTitle(tr("Print Document"));
-   if (dialog->exec() != QDialog::Accepted)
-      return;
-
-   if( rec == 0 )
-      return;
-
-   /* Instantiate the Webview and then connect its signal */
-   connect( doc, SIGNAL(loadFinished(bool)), this, SLOT(loadComplete(bool)) );
-   doc->setHtml(getHTMLFormat());
-}
-
-void RecipeFormatter::printPreview()
+void RecipeFormatter::print(QPrinter* mainPrinter, QPrintDialog *dialog, 
+      int action, QFile* outFile)
 {
    if( rec == 0 )
       return;
 
+   // Short cut if we are saving to HTML
+   if ( action == HTML )
+   {
+      QTextStream out(outFile);
+      out << getHTMLFormat();
+      outFile->close();
+      return;
+   }
+   // We are printing hard copy
+   if ( action == PRINT )
+   {
+      printer = mainPrinter;
+      dialog->setWindowTitle(tr("Print Document"));
+      if (dialog->exec() != QDialog::Accepted)
+         return;
+      connect( doc, SIGNAL(loadFinished(bool)), this, SLOT(loadComplete(bool)) );
+   }
+
    doc->setHtml(getHTMLFormat());
-   docDialog->show();
+   if ( action == PREVIEW )
+      docDialog->show();
 }
+
