@@ -37,6 +37,8 @@
 #include <QDesktopServices>
 #include <QSharedPointer>
 #include <QtNetwork/QNetworkRequest>
+#include <QPixmap>
+#include <QSplashScreen>
 
 #include "brewtarget.h"
 #include "config.h"
@@ -53,6 +55,7 @@
 #include "SIVolumeUnitSystem.h"
 #include "CelsiusTempUnitSystem.h"
 #include "ImperialVolumeUnitSystem.h"
+#include "BtSplashScreen.h"
 
 QApplication* Brewtarget::app;
 MainWindow* Brewtarget::mainWindow;
@@ -65,6 +68,7 @@ bool Brewtarget::userDatabaseDidNotExist = false;
 QDateTime Brewtarget::lastDbMergeRequest = QDateTime::fromString("1986-02-24T06:00:00", Qt::ISODate);
 
 QString Brewtarget::currentLanguage = "en";
+QString Brewtarget::userDataDir = getConfigDir();
 
 bool Brewtarget::checkVersion = true;
 
@@ -125,6 +129,11 @@ bool Brewtarget::ensureDirectoriesExist()
 
 void Brewtarget::checkForNewVersion()
 {
+
+   // Don't do anything if the checkVersion flag was set false
+   if ( checkVersion == false ) 
+      return;
+
    QNetworkAccessManager manager;
    QEventLoop loop;
    QUrl url("http://brewtarget.sourceforge.net/version");
@@ -175,7 +184,48 @@ void Brewtarget::checkForNewVersion()
    }
 }
 
-bool Brewtarget::ensureFilesExist()
+bool Brewtarget::copyDataFiles(QString newPath)
+{
+   QString dbFileName, recipeFileName, mashFileName, optionsFileName;
+   bool success = true;
+
+   // Database files.
+   dbFileName = getUserDataDir() + "database.xml";
+   recipeFileName = getUserDataDir() + "recipes.xml";
+   mashFileName = getUserDataDir() + "mashs.xml";
+   //optionsFileName = getUserDataDir() + "options.xml";
+
+   success &= QFile::copy(dbFileName, newPath + "database.xml");
+   success &= QFile::copy(recipeFileName, newPath + "recipes.xml");
+   success &= QFile::copy(mashFileName, newPath + "mashs.xml");
+   //success &= QFile::copy(optionsFileName, newPath + "options.xml");
+
+   return success;
+}
+
+bool Brewtarget::ensureOptionFileExists()
+{
+   QString optionsFileName;
+   QFile optionsFile;
+   bool success = true;
+
+   optionsFileName = getConfigDir() + "options.xml";
+   optionsFile.setFileName(optionsFileName);
+
+   if( !optionsFile.exists() )
+   {
+      success = QFile::copy(Brewtarget::getDataDir() + "options.xml", optionsFileName);
+      if( ! success )
+      {
+         logE(QString("Could not copy \"%1\" to \"%2\"").arg(Brewtarget::getDataDir() + "options.xml").arg(optionsFileName));
+         return false;
+      }
+   }
+
+   return success;
+}
+
+bool Brewtarget::ensureDataFilesExist()
 {
    QString dbFileName, recipeFileName, mashFileName, optionsFileName, logFileName;
    QFile dbFile, recipeFile, mashFile, optionsFile;
@@ -184,18 +234,16 @@ bool Brewtarget::ensureFilesExist()
    logFile = new QFile();
 
    // Database files.
-   dbFileName = getConfigDir() + "database.xml";
-   recipeFileName = getConfigDir() + "recipes.xml";
-   mashFileName = getConfigDir() + "mashs.xml";
-   optionsFileName = getConfigDir() + "options.xml";
+   dbFileName = getUserDataDir() + "database.xml";
+   recipeFileName = getUserDataDir() + "recipes.xml";
+   mashFileName = getUserDataDir() + "mashs.xml";
    
    dbFile.setFileName(dbFileName);
    recipeFile.setFileName(recipeFileName);
    mashFile.setFileName(mashFileName);
-   optionsFile.setFileName(optionsFileName);
    
    // Log file
-   logFile->setFileName(getConfigDir() + "brewtarget_log.txt");
+   logFile->setFileName(getUserDataDir() + "brewtarget_log.txt");
    if( logFile->open(QFile::WriteOnly | QFile::Truncate) )
       logStream = new QTextStream(logFile);
    else
@@ -241,15 +289,6 @@ bool Brewtarget::ensureFilesExist()
          return false;
       }
    }
-   if( !optionsFile.exists() )
-   {
-      success &= QFile::copy(Brewtarget::getDataDir() + "options.xml", optionsFileName);
-      if( ! success )
-      {
-         logE(QString("Could not copy \"%1\" to \"%2\"").arg(Brewtarget::getDataDir() + "options.xml").arg(optionsFileName));
-         return false;
-      }
-   }
 
    return success;
 }
@@ -309,6 +348,11 @@ iUnitSystem Brewtarget::getVolumeUnitSystem()
    return volumeUnitSystem;
 }
 
+int Brewtarget::getColorUnit()
+{
+   return colorUnit;
+}
+
 TempScale Brewtarget::getTemperatureScale()
 {
    return tempScale;
@@ -328,7 +372,7 @@ QString Brewtarget::getDataDir()
 
 #else // Windows OS.
 
-   dir += "/";
+   dir += "/../data/";
 
 #endif
 
@@ -352,7 +396,7 @@ QString Brewtarget::getDocDir()
 
 #else // Windows OS.
 
-   dir += "/doc/";
+   dir += "/../doc/";
 
 #endif
 
@@ -434,33 +478,71 @@ QString Brewtarget::getConfigDir(bool *success)
 
 #else // Windows OS.
 
-   QString dir= app->applicationDirPath();
-   dir += "/";
+   QDir dir;
+   // Before the app is running, app==0
+   if( app != 0 )
+   {
+      // This is the bin/ directory.
+      dir = QDir(app->applicationDirPath());
+      dir.cdUp();
+   }
+   else
+   {
+      // We're either in bin/ or bin/../
+      dir = QDir::current(); // Guess it's bin/../.
+      if( ! dir.exists("bin") ) // Already in bin/
+         dir.cdUp();
+   }
+   // Now we should be in the base directory (i.e. Brewtarget-1.2.4/)
+
+   dir.cd("data");
    if( success != 0 )
       *success = true;
-   return dir;
+   return dir.absolutePath() + "/";
 
 #endif
 }
 
+QString Brewtarget::getUserDataDir()
+{
+   if( userDataDir.endsWith('/') || userDataDir.endsWith('\\') )
+      return userDataDir;
+   else
+      return userDataDir + "/";
+}
+
 int Brewtarget::run()
 {
    int ret;
    bool success;
-
+   
+   //QPixmap splashImg(BTICON);
+   //QSplashScreen splashScreen(splashImg);
+   BtSplashScreen splashScreen;
+   splashScreen.show();
+   
+   app->processEvents(); // So we can process mouse clicks on splash window.
+   
    success = ensureDirectoriesExist(); // Make sure all the necessary directories are ok.
+   ensureOptionFileExists();
+   readPersistentOptions(); // Read all the options for bt.
+
    if( success )
-      success = ensureFilesExist(); // Make sure all the files we need exist before starting.
+      success = ensureDataFilesExist(); // Make sure all the files we need exist before starting.
    if( ! success )
       return 1;
 
-   readPersistentOptions(); // Read all the options for bt.
    loadTranslations(); // Do internationalization.
 
+   app->processEvents();
+   
+   splashScreen.showMessage("Loading...");
    Database::initialize();
    
    mainWindow = new MainWindow();
    mainWindow->setVisible(true);
+   
+   splashScreen.finish(mainWindow);
 
    checkForNewVersion();
 
@@ -539,7 +621,13 @@ QString Brewtarget::displayAmount( double amount, Unit* units, int precision )
    return ret;
 }
 
-QString Brewtarget::displayThickness(double thick_lkg)
+void Brewtarget::getThicknessUnits( Unit** volumeUnit, Unit** weightUnit )
+{
+   *volumeUnit = volumeSystem->thicknessUnit();
+   *weightUnit = weightSystem->thicknessUnit();
+}
+
+QString Brewtarget::displayThickness( double thick_lkg, bool showUnits )
 {
    int fieldWidth = 0;
    char format = 'f';
@@ -551,7 +639,37 @@ QString Brewtarget::displayThickness(double thick_lkg)
    double num = volUnit->fromSI(thick_lkg);
    double den = weightUnit->fromSI(1.0);
 
-   return QString("%1 %2/%3").arg(num/den, fieldWidth, format, precision).arg(volUnit->getUnitName()).arg(weightUnit->getUnitName());
+   if( showUnits )
+      return QString("%1 %2/%3").arg(num/den, fieldWidth, format, precision).arg(volUnit->getUnitName()).arg(weightUnit->getUnitName());
+   else
+      return QString("%1").arg(num/den, fieldWidth, format, precision).arg(volUnit->getUnitName()).arg(weightUnit->getUnitName());
+}
+
+QString Brewtarget::getOptionValue(const QDomDocument& optionsDoc, const QString& option, bool* hasOption)
+{
+   QDomNode node, child;
+   QDomText textNode;
+   QDomNodeList list;
+
+   list = optionsDoc.elementsByTagName(option);
+   if(list.length() <= 0)
+   {
+      Brewtarget::logW(QString("Could not find the <%1> tag in the option file.").arg(option));
+      if( hasOption != 0 )
+         *hasOption = false;
+      return "";
+   }
+   else
+   {
+      node = list.at(0);
+      child = node.firstChild();
+      textNode = child.toText();
+
+      if( hasOption != 0 )
+         *hasOption = true;
+
+      return textNode.nodeValue();
+   }
 }
 
 void Brewtarget::readPersistentOptions()
@@ -559,13 +677,11 @@ void Brewtarget::readPersistentOptions()
    QFile xmlFile(getConfigDir() + "options.xml");
    optionsDoc = new QDomDocument();
    QDomElement root;
-   QDomNode node, child;
-   QDomText textNode;
-   QDomNodeList list;
    QString err;
    QString text;
    int line;
    int col;
+   bool hasOption;
 
    // Try to open xmlFile.
    if( ! xmlFile.open(QIODevice::ReadOnly) )
@@ -581,104 +697,32 @@ void Brewtarget::readPersistentOptions()
 
    root = optionsDoc->documentElement();
 
-   // backwards compat for old 'english units' - these were US Customary
-   // for weight and volume and fahrenheit for temp
-   list = optionsDoc->elementsByTagName(QString("english_units"));
-   if( list.length() <= 0 )
-   {
-      //could be pre 'english_units', in which case default to US Customary
-      weightUnitSystem = USCustomary;
-      volumeUnitSystem = USCustomary;
-      tempScale = Fahrenheit;
-
-      weightSystem = UnitSystems::usWeightUnitSystem();
-      volumeSystem = UnitSystems::usVolumeUnitSystem();
-      tempSystem = UnitSystems::fahrenheitTempUnitSystem();
-   }
-   else
-   {
-      node = list.at(0);
-      child = node.firstChild();
-      textNode = child.toText();
-      text = textNode.nodeValue();
-
-      if( text == "true" )
-      {
-         weightUnitSystem = USCustomary;
-         volumeUnitSystem = USCustomary;
-         tempScale = Fahrenheit;
-
-         weightSystem = UnitSystems::usWeightUnitSystem();
-         volumeSystem = UnitSystems::usVolumeUnitSystem();
-         tempSystem = UnitSystems::fahrenheitTempUnitSystem();
-      }
-      else
-      {
-         weightUnitSystem = SI;
-         volumeUnitSystem = SI;
-         tempScale = Celsius;
-
-         weightSystem = UnitSystems::siWeightUnitSystem();
-         volumeSystem = UnitSystems::siVolumeUnitSystem();
-         tempSystem = UnitSystems::celsiusTempUnitSystem();
-      }
- 
-   }
-
    //================Version Checking========================
-   list = optionsDoc->elementsByTagName(QString("check_version"));
-   if( list.length() > 0 )
-   {
-      node = list.at(0);
-      child = node.firstChild();
-      textNode = child.toText();
-      text = textNode.nodeValue();
-
-      if( text == "true" )
-         checkVersion = true;
-      else
-         checkVersion = false;
-   }
+   text = getOptionValue(*optionsDoc, "check_version");
+   if( text == "true" )
+      checkVersion = true;
+   else
+      checkVersion = false;
 
    //=====================Last DB Merge Request======================
-   list = optionsDoc->elementsByTagName(QString("last_db_merge_req"));
-   if( list.length() > 0 )
-   {
-      node = list.at(0);
-      child = node.firstChild();
-      textNode = child.toText();
-      text = textNode.nodeValue();
-
+   text = getOptionValue(*optionsDoc, "last_db_merge_req", &hasOption);
+   if( hasOption )
       lastDbMergeRequest = QDateTime::fromString(text, Qt::ISODate);
-   }
 
    //=====================Language====================
-   list = optionsDoc->elementsByTagName(QString("language"));
-   if(list.length() <= 0)
-      Brewtarget::logW(QString("Could not find the language tag in the option file."));
-   else
-   {
-      node = list.at(0);
-      child = node.firstChild();
-      textNode = child.toText();
-      text = textNode.nodeValue();
-
+   text = getOptionValue(*optionsDoc, "language", &hasOption);
+   if( hasOption )
       setLanguage(text);
-   }
+
+   //=======================Data Dir===========================
+   text = getOptionValue(*optionsDoc, "user_data_dir", &hasOption);
+   if( hasOption )
+      userDataDir = text;
 
    //=======================Weight=====================
-   list = optionsDoc->elementsByTagName(QString("weight_unit_system"));
-   if( list.length() <= 0 )
+   text = getOptionValue(*optionsDoc, "weight_unit_system", &hasOption);
+   if( hasOption )
    {
-      Brewtarget::log(Brewtarget::ERROR, QString("Could not find the weight_unit_system tag in the option file."));
-   }
-   else
-   {
-      node = list.at(0);
-      child = node.firstChild();
-      textNode = child.toText();
-      text = textNode.nodeValue();
-
       if( text == "Imperial" )
       {
          weightUnitSystem = Imperial;
@@ -697,77 +741,50 @@ void Brewtarget::readPersistentOptions()
    }
 
    //===========================Volume=======================
-   list = optionsDoc->elementsByTagName(QString("volume_unit_system"));
-   if( list.length() <= 0 )
+   text = getOptionValue(*optionsDoc, "volume_unit_system", &hasOption);
+   if( hasOption )
    {
-      Brewtarget::log(Brewtarget::ERROR, QString("Could not find the volume_unit_system tag in the option file."));
-   }
-   else
-   {
-      node = list.at(0);
-      child = node.firstChild();
-      textNode = child.toText();
-      text = textNode.nodeValue();
-
       if( text == "Imperial" )
       {
          volumeUnitSystem = Imperial;
-	 volumeSystem = UnitSystems::imperialVolumeUnitSystem();
+         volumeSystem = UnitSystems::imperialVolumeUnitSystem();
       }
       else if (text == "USCustomary")
       {
          volumeUnitSystem = USCustomary;
-	 volumeSystem = UnitSystems::usVolumeUnitSystem();
+         volumeSystem = UnitSystems::usVolumeUnitSystem();
       }
       else
       {
          volumeUnitSystem = SI;
-	 volumeSystem = UnitSystems::siVolumeUnitSystem();
+         volumeSystem = UnitSystems::siVolumeUnitSystem();
       }
    }
 
    //=======================Temp======================
-   list = optionsDoc->elementsByTagName(QString("temperature_scale"));
-   if( list.length() <= 0 )
+   text = getOptionValue(*optionsDoc, "temperature_scale", &hasOption);
+   if( hasOption )
    {
-      Brewtarget::log(Brewtarget::ERROR, QString("Could not find the temperature_scale tag in the option file."));
-   }
-   else
-   {
-      node = list.at(0);
-      child = node.firstChild();
-      textNode = child.toText();
-      text = textNode.nodeValue();
-
       if( text == "Fahrenheit" )
       {
          tempScale = Fahrenheit;
-	 tempSystem = UnitSystems::fahrenheitTempUnitSystem();
+         tempSystem = UnitSystems::fahrenheitTempUnitSystem();
       }
       else
       {
          tempScale = Celsius;
-	 tempSystem = UnitSystems::celsiusTempUnitSystem();
+         tempSystem = UnitSystems::celsiusTempUnitSystem();
       }
    }
 
+   //======================Time======================
    // Set the one and only time system.
    timeSystem = UnitSystems::timeUnitSystem();
 
    //===================IBU===================
-   // Get IBU formula.
-   list = optionsDoc->elementsByTagName(QString("ibu_formula"));
-   if( list.length() <= 0 )
+   text = getOptionValue(*optionsDoc, "ibu_formula", &hasOption);
+   if( hasOption )
    {
-      Brewtarget::log(Brewtarget::ERROR, QString("Could not find the ibu_formula tag in the option file."));
-   }
-   else
-   {
-      node = list.at(0);
-      child = node.firstChild();
-      textNode = child.toText();
-      text = textNode.nodeValue();
-
       if( text == "tinseth" )
          ibuFormula = TINSETH;
       else if( text == "rager" )
@@ -779,19 +796,9 @@ void Brewtarget::readPersistentOptions()
    }
 
    //========================Color======================
-   // Get color formula.
-   list = optionsDoc->elementsByTagName(QString("color_formula"));
-   if( list.length() <= 0 )
+   text = getOptionValue(*optionsDoc, "color_formula", &hasOption);
+   if( hasOption )
    {
-      Brewtarget::log(Brewtarget::ERROR, QString("Could not find the color_formula tag in the option file."));
-   }
-   else
-   {
-      node = list.at(0);
-      child = node.firstChild();
-      textNode = child.toText();
-      text = textNode.nodeValue();
-
       if( text == "morey" )
          colorFormula = MOREY;
       else if( text == "daniel" )
@@ -805,17 +812,9 @@ void Brewtarget::readPersistentOptions()
    }
 
    //========================Gravity==================
-   // Get plato usage.
-   list = optionsDoc->elementsByTagName(QString("use_plato"));
-   if( list.length() <= 0 )
-      Brewtarget::logW( "Could not find the use_plato tag in the option file." );
-   else
+   text = getOptionValue(*optionsDoc, "use_plato", &hasOption);
+   if( hasOption )
    {
-      node = list.at(0);
-      child = node.firstChild();
-      textNode = child.toText();
-      text = textNode.nodeValue();
-
       if( text == "true" )
          usePlato = true;
       else if( text == "false" )
@@ -827,22 +826,15 @@ void Brewtarget::readPersistentOptions()
    }
 
    //=======================Color unit===================
-   list = optionsDoc->elementsByTagName("color_unit");
-   if( list.length() <= 0 )
-      Brewtarget::logW( "Could not find the color_unit tag in the option file." );
-   else
+   text = getOptionValue(*optionsDoc, "color_unit", &hasOption);
+   if( hasOption )
    {
-      node = list.at(0);
-      child = node.firstChild();
-      textNode = child.toText();
-      text = textNode.nodeValue();
-
       if( text == "srm" )
          colorUnit = SRM;
       else if( text == "ebc" )
          colorUnit = EBC;
       else
-         Brewtarget::logW(QString("Bad color_unit type: %1").arg(text));;
+         Brewtarget::logW(QString("Bad color_unit type: %1").arg(text));
    }
 
    delete optionsDoc;
@@ -878,6 +870,12 @@ void Brewtarget::savePersistentOptions()
    node.appendChild(child);
    root.appendChild(node);
 
+   // User data dir.
+   node = optionsDoc->createElement("user_data_dir");
+   child = optionsDoc->createTextNode( userDataDir );
+   node.appendChild(child);
+   root.appendChild(node);
+
    // Unit Systems.
    node = optionsDoc->createElement("weight_unit_system");
    child = optionsDoc->createTextNode( unitSystemToString(weightUnitSystem) );
@@ -986,11 +984,28 @@ double Brewtarget::tempQStringToSI(QString qstr)
    return tempSystem->qstringToSI(qstr);
 }
 
+double Brewtarget::colorQStringToSI(QString qstr)
+{
+   if ( colorUnit == SRM )
+      return qstr.toDouble();
+   else
+      return qstr.toDouble() * 12.7/25.0;
+}
+
 double Brewtarget::timeQStringToSI(QString qstr)
 {
    return timeSystem->qstringToSI(qstr);
 }
 
+bool Brewtarget::hasUnits(QString qstr)
+{
+   // accepts X.YZ as well as .YZ followed by some unit string
+   QRegExp amtUnit("(\\d+(?:\\.\\d+)?|\\.\\d+)\\s*(\\w+)?");
+   amtUnit.indexIn(qstr);
+
+   return amtUnit.cap(2).size() > 0;
+}
+
 QString Brewtarget::displayOG( double og, bool showUnits )
 {
    QString tmp = (showUnits & usePlato) ? "%1 %2" : "%1";
