@@ -97,9 +97,9 @@ void Database::initialize()
    int col;
    unsigned int i, size;
 
-   dbFileName = (Brewtarget::getConfigDir() + "database.xml");
-   recipeFileName = (Brewtarget::getConfigDir() + "recipes.xml");
-   mashFileName = (Brewtarget::getConfigDir() + "mashs.xml");
+   dbFileName = (Brewtarget::getUserDataDir() + "database.xml");
+   recipeFileName = (Brewtarget::getUserDataDir() + "recipes.xml");
+   mashFileName = (Brewtarget::getUserDataDir() + "mashs.xml");
 
    dbFile.setFileName(dbFileName);
    recipeFile.setFileName(recipeFileName);
@@ -273,7 +273,7 @@ void Database::initialize()
 
 void Database::mergeBeerXMLRecDocs( QDomDocument& first, const QDomDocument& last )
 {
-   QDomNode root = first.firstChild();
+  QDomNode root = first.elementsByTagName("RECIPES").at(0);;
    QDomNodeList list;
    int i, size;
 
@@ -286,7 +286,7 @@ void Database::mergeBeerXMLRecDocs( QDomDocument& first, const QDomDocument& las
 
 void Database::mergeBeerXMLDBDocs( QDomDocument& first, const QDomDocument& last )
 {
-   QDomNode root = first.firstChild();
+   QDomNode root = first.elementsByTagName("DATABASE").at(0);
    QDomNodeList list;
    int i, size;
 
@@ -336,6 +336,7 @@ void Database::mergeBeerXMLDBDocs( QDomDocument& first, const QDomDocument& last
    size = list.size();
    for( i = 0; i < size; ++i )
       root.appendChild(list.at(i));
+   
 }
 
 void Database::resortAll()
@@ -671,7 +672,7 @@ void Database::addRecipe(Recipe* rec, bool copySubelements)
          addYeast( rec->getYeast(i), true );
    }
 
-   hasChanged(DBALL);
+   hasChanged(DBRECIPE);
 }
 
 void Database::addStyle(Style* style, bool disableNotify)
@@ -693,60 +694,141 @@ void Database::removeEquipment(Equipment* equip)
    hasChanged(QVariant(DBEQUIP));
 }
 
+void Database::removeEquipment(QList<Equipment*> equip)
+{
+    foreach (Equipment* doa, equip)
+        equipments.removeOne(doa);
+    if ( equip.count() )
+        hasChanged(QVariant(DBEQUIP));
+}
+
 void Database::removeFermentable(Fermentable* ferm)
 {
    fermentables.removeOne(ferm);
    hasChanged(QVariant(DBFERM));
 }
 
+void Database::removeFermentable(QList<Fermentable*> ferm)
+{
+   foreach (Fermentable* doa, ferm)
+      fermentables.removeOne(doa);
+   if (ferm.count())
+       hasChanged(QVariant(DBFERM));
+}
+
 void Database::removeHop(Hop* hop)
 {
    hops.removeOne(hop);
    hasChanged(QVariant(DBHOP));
 }
 
+void Database::removeHop(QList<Hop*> hop)
+{
+   foreach (Hop* doa, hop)
+       hops.removeOne(doa);
+
+   if (hop.count())
+       hasChanged(QVariant(DBHOP));
+}
+
 void Database::removeMash(Mash* mash)
 {
    mashs.removeOne(mash);
    hasChanged(QVariant(DBMASH));
 }
 
+void Database::removeMash(QList<Mash*> mash)
+{
+   foreach( Mash* doa, mash)
+       mashs.removeOne(doa);
+   if (mash.count())
+       hasChanged(QVariant(DBMASH));
+}
+
 void Database::removeMashStep(MashStep* mashStep)
 {
    mashSteps.removeOne(mashStep);
    hasChanged(QVariant(DBMASHSTEP));
 }
 
+void Database::removeMashStep(QList<MashStep*> mashStep)
+{
+   foreach( MashStep* doa, mashStep)
+       mashSteps.removeOne(doa);
+   if (mashStep.count())
+       hasChanged(QVariant(DBMASHSTEP));
+}
+
 void Database::removeMisc(Misc* misc)
 {
    miscs.removeOne(misc);
    hasChanged(QVariant(DBMISC));
 }
 
+void Database::removeMisc(QList<Misc*> misc)
+{
+   foreach( Misc* doa, misc)
+       miscs.removeOne(doa);
+   if (misc.count())
+       hasChanged(QVariant(DBMISC));
+}
+
 void Database::removeRecipe(Recipe* rec)
 {
    recipes.removeOne(rec);
    hasChanged(QVariant(DBRECIPE));
 }
 
+void Database::removeRecipe(QList<Recipe*> rec)
+{
+   foreach( Recipe *doa, rec)
+       recipes.removeOne(doa);
+   if (rec.count())
+       hasChanged(QVariant(DBRECIPE));
+}
+
 void Database::removeStyle(Style* style)
 {
    styles.removeOne(style); // Wow, that was easy.
    hasChanged(QVariant(DBSTYLE));
 }
 
+void Database::removeStyle(QList<Style*> style)
+{
+   foreach (Style* doa, style)
+       styles.removeOne(doa); // Wow, that was easy.
+   if (style.count())
+       hasChanged(QVariant(DBSTYLE));
+}
+
 void Database::removeWater(Water* water)
 {
    waters.removeOne(water);
    hasChanged(QVariant(DBWATER));
 }
 
+void Database::removeWater(QList<Water*> water)
+{
+   foreach(Water* doa, water)
+       waters.removeOne(doa);
+   if (water.count())
+       hasChanged(QVariant(DBWATER));
+}
+
 void Database::removeYeast(Yeast* yeast)
 {
    yeasts.removeOne(yeast);
    hasChanged(QVariant(DBYEAST));
 }
 
+void Database::removeYeast(QList<Yeast*> yeast)
+{
+   foreach( Yeast* doa, yeast)
+       yeasts.removeOne(doa);
+   if (yeast.count())
+       hasChanged(QVariant(DBYEAST));
+}
+
 void Database::addWater(Water* water, bool disableNotify)
 {
    if( water != 0 )
@@ -1064,3 +1146,13 @@ Yeast* Database::findYeastByName(QString name)
 
    return 0;
 }
+
+QString Database::getDbFileName()
+{
+   return dbFileName;
+}
+
+QString Database::getRecipeFileName()
+{
+   return recipeFileName;
+}
\ No newline at end of file
