@@ -145,7 +145,13 @@ void Recipe::toXml(QDomDocument& doc, QDomNode& parent)
    for( i = 0; i < size; ++i )
       instructions[i]->toXml(doc, tmpNode);
    recipeNode.appendChild(tmpNode);
-   
+
+   tmpNode = doc.createElement("BREWNOTES");
+   size = brewNotes.size();
+   for(i=0; i < size; ++i)
+      brewNotes[i]->toXml(doc,tmpNode);
+   recipeNode.appendChild(tmpNode);
+
    tmpNode = doc.createElement("ASST_BREWER");
    tmpText = doc.createTextNode(asstBrewer);
    tmpNode.appendChild(tmpText);
@@ -278,6 +284,7 @@ void Recipe::setDefaults()
    miscs = QVector<Misc*>();
    yeasts = QVector<Yeast*>();
    waters = QVector<Water*>();
+   brewNotes = QVector<BrewNote*>();
    mash = new Mash();
    addObserved(mash);
 
@@ -468,6 +475,12 @@ void Recipe::fromNode(const QDomNode& recipeNode)
          for( instructionNode = child; ! instructionNode.isNull(); instructionNode = instructionNode.nextSibling() )
             addInstruction(new Instruction(instructionNode));
       }
+      else if( property == "BREWNOTES" )
+      {
+         QDomNode bnNode;
+         for( bnNode = child; ! bnNode.isNull(); bnNode = bnNode.nextSibling() )
+            addBrewNote(new BrewNote(this,bnNode));
+      }
       else if( property == "MASH" )
       {
          setMash(new Mash(node));
@@ -735,9 +748,9 @@ Instruction* Recipe::getInstruction(unsigned int i)
 
 Instruction* Recipe::getMashFermentable() const
 {
-	Instruction* ins;
-	QString str,tmp;
-	unsigned int i;
+   Instruction* ins;
+   QString str,tmp;
+   unsigned int i;
 
     /*** Add grains ***/
      ins = new Instruction();
@@ -746,11 +759,11 @@ Instruction* Recipe::getMashFermentable() const
      for( i = 0; static_cast<int>(i) < fermentables.size(); ++i )
      {
         if( fermentables[i]->getIsMashed() )
-			 tmp = QString("%1 %2, ")
-				.arg(Brewtarget::displayAmount(fermentables[i]->getAmount_kg(), Units::kilograms))
-				.arg(fermentables[i]->getName());
-		 	 str += tmp;
-		 	 ins->setReagent(tmp);
+          tmp = QString("%1 %2, ")
+            .arg(Brewtarget::displayAmount(fermentables[i]->getAmount_kg(), Units::kilograms))
+            .arg(fermentables[i]->getName());
+           str += tmp;
+           ins->setReagent(tmp);
      }
      str += QObject::tr("to the mash tun.");
      ins->setDirections(str);
@@ -761,10 +774,10 @@ Instruction* Recipe::getMashFermentable() const
 
 Instruction* Recipe::getMashWater(unsigned int size) const
 {
-	Instruction* ins;
-	MashStep* mstep;
-	QString str, tmp;
-	unsigned int i;
+   Instruction* ins;
+   MashStep* mstep;
+   QString str, tmp;
+   unsigned int i;
 
     ins = new Instruction();
     ins->setName(QObject::tr("Heat water"));
@@ -778,8 +791,8 @@ Instruction* Recipe::getMashWater(unsigned int size) const
        tmp = QObject::tr("%1 water to %2, ")
               .arg(Brewtarget::displayAmount(mstep->getInfuseAmount_l(), Units::liters))
               .arg(Brewtarget::displayAmount(mstep->getInfuseTemp_c(), Units::celsius));
-		 str += tmp;
-		 ins->setReagent(tmp);
+       str += tmp;
+       ins->setReagent(tmp);
     }
     str += QObject::tr("for upcoming infusions.");
     ins->setDirections(str);
@@ -789,13 +802,13 @@ Instruction* Recipe::getMashWater(unsigned int size) const
 
 QVector<PreInstruction> Recipe::getMashInstructions(double timeRemaining, double totalWaterAdded_l, unsigned int size) const
 {
-	QVector<PreInstruction> preins;
-	MashStep* mstep;
-	QString str;
+   QVector<PreInstruction> preins;
+   MashStep* mstep;
+   QString str;
 
-	unsigned int i;
+   unsigned int i;
 
-	preins.clear();
+   preins.clear();
     for( i = 0; i < size; ++i )
     {
        mstep = mash->getMashStep(i);
@@ -831,18 +844,18 @@ QVector<PreInstruction> Recipe::getMashInstructions(double timeRemaining, double
 
 QVector<PreInstruction> Recipe::getHopSteps(Hop::Use type) const
 {
-	QVector<PreInstruction> preins;
-	QString str;
-	unsigned int i;
+   QVector<PreInstruction> preins;
+   QString str;
+   unsigned int i;
 
-	preins.clear();
+   preins.clear();
 
     for( i = 0; static_cast<int>(i) < hops.size(); ++i )
     {
        Hop* hop = hops[i];
        if( hop->getUse() == type )
        {
-          if( type == Hop::USEAROMA  || type == Hop::USEBOIL )
+          if( type == Hop::USEBOIL )
              str = QObject::tr("Put %1 %2 into boil for %3.");
           else if( type == Hop::USEDRY_HOP )
              str = QObject::tr("Put %1 %2 into fermenter for %3.");
@@ -850,6 +863,8 @@ QVector<PreInstruction> Recipe::getHopSteps(Hop::Use type) const
              str = QObject::tr("Put %1 %2 into first wort for %3.");
           else if( type == Hop::USEMASH )
              str = QObject::tr("Put %1 %2 into mash for %3.");
+          else if( type == Hop::USEAROMA )
+             str = QObject::tr("Steep %1 %2 in wort for %3.");
           else
           {
              Brewtarget::logW("Recipe::getHopSteps(): Unrecognized hop use.");
@@ -863,14 +878,14 @@ QVector<PreInstruction> Recipe::getHopSteps(Hop::Use type) const
           preins.push_back(PreInstruction(str, QObject::tr("Hop addition"), hop->getTime_min()));
        }
     }
-	return preins;
+   return preins;
 }
 
 QVector<PreInstruction> Recipe::getMiscSteps(Misc::Use type) const
 {
-	QVector<PreInstruction> preins;
-	QString str;
-	unsigned int i;
+   QVector<PreInstruction> preins;
+   QString str;
+   unsigned int i;
 
     for( i = 0; static_cast<int>(i) < miscs.size(); ++i )
     {
@@ -914,92 +929,92 @@ Instruction* Recipe::getFirstWortHops() const
 
    for( i = 0; static_cast<int>(i) < hops.size(); ++i )
    {
-	  Hop* hop = hops[i];
-	  if( hop->getUse() == Hop::USEFIRST_WORT )
-	  {
-		 tmp = QString("%1 %2,")
-				.arg(Brewtarget::displayAmount(hop->getAmount_kg(), Units::kilograms))
-				.arg(hop->getName());
-		 str += tmp;
-		 hasHop = true;
-	  }
+     Hop* hop = hops[i];
+     if( hop->getUse() == Hop::USEFIRST_WORT )
+     {
+       tmp = QString("%1 %2,")
+            .arg(Brewtarget::displayAmount(hop->getAmount_kg(), Units::kilograms))
+            .arg(hop->getName());
+       str += tmp;
+       hasHop = true;
+     }
    }
    str += ".";
    if( hasHop )
    {
-	  ins = new Instruction();
-	  ins->setName(QObject::tr("First wort hopping"));
-	  ins->setDirections(str);
-	  ins->setReagent(tmp);
-	  return ins;
+     ins = new Instruction();
+     ins->setName(QObject::tr("First wort hopping"));
+     ins->setDirections(str);
+     ins->setReagent(tmp);
+     return ins;
    }
    return 0;
 }
 
 Instruction* Recipe::getTopOff() const
 {
-	double wortInBoil_l = 0.0;
-	QString str,tmp;
-	Instruction* ins;
+   double wortInBoil_l = 0.0;
+   QString str,tmp;
+   Instruction* ins;
 
-	if( equipment != 0 )
-	{
-		wortInBoil_l = estimateWortFromMash_l() - equipment->getLauterDeadspace_l();
-		str = QObject::tr("You should now have %1 wort.")
-			.arg(Brewtarget::displayAmount( wortInBoil_l, Units::liters));
-		if ( equipment->getTopUpKettle_l() != 0 )
-		{
-			wortInBoil_l += equipment->getTopUpKettle_l();
-			tmp = QObject::tr(" Add %1 water to the kettle, bringing pre-boil volume to %2.")
-				.arg(Brewtarget::displayAmount(equipment->getTopUpKettle_l(), Units::liters))
-				.arg(Brewtarget::displayAmount(wortInBoil_l, Units::liters));
+   if( equipment != 0 )
+   {
+      wortInBoil_l = estimateWortFromMash_l() - equipment->getLauterDeadspace_l();
+      str = QObject::tr("You should now have %1 wort.")
+         .arg(Brewtarget::displayAmount( wortInBoil_l, Units::liters));
+      if ( equipment->getTopUpKettle_l() != 0 )
+      {
+         wortInBoil_l += equipment->getTopUpKettle_l();
+         tmp = QObject::tr(" Add %1 water to the kettle, bringing pre-boil volume to %2.")
+            .arg(Brewtarget::displayAmount(equipment->getTopUpKettle_l(), Units::liters))
+            .arg(Brewtarget::displayAmount(wortInBoil_l, Units::liters));
 
-			str += tmp;
+         str += tmp;
 
-			ins = new Instruction();
-			ins->setName(QObject::tr("Pre-boil"));
-			ins->setDirections(str);
-			ins->setReagent(tmp);
-			return ins;
-		}
-	}
-	return 0;
+         ins = new Instruction();
+         ins->setName(QObject::tr("Pre-boil"));
+         ins->setDirections(str);
+         ins->setReagent(tmp);
+         return ins;
+      }
+   }
+   return 0;
 }
 
 bool Recipe::hasBoilFermentable()
 {
-	unsigned int i;
-	for ( i = 0; static_cast<int>(i) < fermentables.size(); ++i )
-	{
-		Fermentable* ferm = fermentables[i];
-		if( ferm->getIsMashed() || ferm->getAddAfterBoil() )
-			continue;
-		return true;
-	}
-	return false;
+   unsigned int i;
+   for ( i = 0; static_cast<int>(i) < fermentables.size(); ++i )
+   {
+      Fermentable* ferm = fermentables[i];
+      if( ferm->getIsMashed() || ferm->getAddAfterBoil() )
+         continue;
+      return true;
+   }
+   return false;
 }
 
 PreInstruction Recipe::getBoilFermentables(double timeRemaining) const
 {
-	bool hasFerms = false;
-	QString str;
-	unsigned int i;
+   bool hasFerms = false;
+   QString str;
+   unsigned int i;
 
-	str = QObject::tr("Boil or steep ");
-	for( i = 0; static_cast<int>(i) < fermentables.size(); ++i )
-	{
-	  Fermentable* ferm = fermentables[i];
-	  if( ferm->getIsMashed() || ferm->getAddAfterBoil() )
-		 continue;
+   str = QObject::tr("Boil or steep ");
+   for( i = 0; static_cast<int>(i) < fermentables.size(); ++i )
+   {
+     Fermentable* ferm = fermentables[i];
+     if( ferm->getIsMashed() || ferm->getAddAfterBoil() )
+       continue;
 
-	  hasFerms = true;
-	  str += QString("%1 %2, ")
-			 .arg(Brewtarget::displayAmount(ferm->getAmount_kg(), Units::kilograms))
-			 .arg(ferm->getName());
-	}
-	str += ".";
+     hasFerms = true;
+     str += QString("%1 %2, ")
+          .arg(Brewtarget::displayAmount(ferm->getAmount_kg(), Units::kilograms))
+          .arg(ferm->getName());
+   }
+   str += ".";
 
-	return PreInstruction(str, QObject::tr("Boil/steep fermentables"), timeRemaining);
+   return PreInstruction(str, QObject::tr("Boil/steep fermentables"), timeRemaining);
 }
 
 Instruction* Recipe::getPostboilFermentables()
@@ -1034,40 +1049,40 @@ Instruction* Recipe::getPostboilFermentables()
    }
    else
    {
-	   return 0;
+      return 0;
    }
 }
 
 Instruction* Recipe::getPostboilSteps()
 {
-	QString str;
-	Instruction* ins;
-	double wort_l = 0.0;
-	double wortInBoil_l = 0.0;
+   QString str;
+   Instruction* ins;
+   double wort_l = 0.0;
+   double wortInBoil_l = 0.0;
 
-	if( equipment != 0 )
-	{
-		wortInBoil_l = estimateWortFromMash_l() - equipment->getLauterDeadspace_l();
-		if ( equipment->getTopUpKettle_l() != 0 )
-			wortInBoil_l += equipment->getTopUpKettle_l();
+   if( equipment != 0 )
+   {
+      wortInBoil_l = estimateWortFromMash_l() - equipment->getLauterDeadspace_l();
+      if ( equipment->getTopUpKettle_l() != 0 )
+         wortInBoil_l += equipment->getTopUpKettle_l();
 
-		wort_l = equipment->wortEndOfBoil_l(wortInBoil_l);
-		str = QObject::tr("You should have %1 wort post-boil.")
-				.arg(Brewtarget::displayAmount( wort_l, Units::liters));
-		str += QObject::tr("\nYou anticipate losing %1 to trub and chiller loss.")
-				.arg(Brewtarget::displayAmount( equipment->getTrubChillerLoss_l(), Units::liters));
-		wort_l -= equipment->getTrubChillerLoss_l();
-		if( equipment->getTopUpWater_l() > 0.0 )
-			 str += QObject::tr("\nAdd %1 top up water into primary.")
-				   .arg(Brewtarget::displayAmount( equipment->getTopUpWater_l(), Units::liters));
-		wort_l += equipment->getTopUpWater_l();
-		str += QObject::tr("\nThe final volume in the primary is %1.")
-				 .arg(Brewtarget::displayAmount(wort_l, Units::liters));
+      wort_l = equipment->wortEndOfBoil_l(wortInBoil_l);
+      str = QObject::tr("You should have %1 wort post-boil.")
+            .arg(Brewtarget::displayAmount( wort_l, Units::liters));
+      str += QObject::tr("\nYou anticipate losing %1 to trub and chiller loss.")
+            .arg(Brewtarget::displayAmount( equipment->getTrubChillerLoss_l(), Units::liters));
+      wort_l -= equipment->getTrubChillerLoss_l();
+      if( equipment->getTopUpWater_l() > 0.0 )
+          str += QObject::tr("\nAdd %1 top up water into primary.")
+               .arg(Brewtarget::displayAmount( equipment->getTopUpWater_l(), Units::liters));
+      wort_l += equipment->getTopUpWater_l();
+      str += QObject::tr("\nThe final volume in the primary is %1.")
+             .arg(Brewtarget::displayAmount(wort_l, Units::liters));
 
-		ins = new Instruction();
-		ins->setName(QObject::tr("Post boil"));
-		ins->setDirections(str);
-		return ins;
+      ins = new Instruction();
+      ins->setName(QObject::tr("Post boil"));
+      ins->setDirections(str);
+      return ins;
    }
    else
    {
@@ -1077,8 +1092,8 @@ Instruction* Recipe::getPostboilSteps()
 
 void Recipe::addPreinstructions( QVector<PreInstruction> preins )
 {
-	unsigned int i;
-	Instruction* ins;
+   unsigned int i;
+   Instruction* ins;
 
     // Add instructions in descending mash time order.
     qSort( preins.begin(), preins.end(), qGreater<PreInstruction>() );
@@ -1108,39 +1123,39 @@ void Recipe::generateInstructions()
 
    if( mash != 0 && mash->getNumMashSteps() > 0 )
    {
-	  size = mash->getNumMashSteps();
+     size = mash->getNumMashSteps();
 
-	  /*** prepare mashed fermentables ***/
-	  instructions += getMashFermentable();
+     /*** prepare mashed fermentables ***/
+     instructions += getMashFermentable();
 
-	  /*** Prepare water additions ***/
-	  instructions += getMashWater(size);
+     /*** Prepare water additions ***/
+     instructions += getMashWater(size);
 
-	  timeRemaining = mash->getTotalTime();
+     timeRemaining = mash->getTotalTime();
 
-	  /*** Generate the mash instructions ***/
-	  preinstructions = getMashInstructions(timeRemaining, totalWaterAdded_l, size);
+     /*** Generate the mash instructions ***/
+     preinstructions = getMashInstructions(timeRemaining, totalWaterAdded_l, size);
 
       /*** Hops mash additions ***/
-	  preinstructions += getHopSteps(Hop::USEMASH);
+     preinstructions += getHopSteps(Hop::USEMASH);
 
       /*** Misc mash additions ***/
-	  preinstructions += getMiscSteps(Misc::USEMASH);
+     preinstructions += getMiscSteps(Misc::USEMASH);
 
-	  /*** Add the preinstructions into the instructions ***/
-	  addPreinstructions(preinstructions);
+     /*** Add the preinstructions into the instructions ***/
+     addPreinstructions(preinstructions);
 
    } // END mash instructions.
 
    // First wort hopping
    ins = getFirstWortHops();
    if ( ins )
-	   instructions += ins;
+      instructions += ins;
 
    // Need to top up the kettle before boil?
    ins = getTopOff();
    if ( ins )
-	   instructions += ins;
+      instructions += ins;
 
    // Boil instructions
    preinstructions.clear();   
@@ -1164,7 +1179,7 @@ void Recipe::generateInstructions()
    
    /*** Get fermentables we haven't added yet ***/
    if ( hasBoilFermentable() )
-	   preinstructions.push_back(getBoilFermentables(timeRemaining));
+      preinstructions.push_back(getBoilFermentables(timeRemaining));
    
    /*** Boiled hops ***/
    preinstructions += getHopSteps(Hop::USEBOIL);
@@ -1177,18 +1192,27 @@ void Recipe::generateInstructions()
    // Add instructions in descending mash time order.
    addPreinstructions(preinstructions);
 
+   // FLAMEOUT
+   instructions += new Instruction(QObject::tr("Flameout"),
+                                   QObject::tr("Stop boiling the wort."));
+
+   // Steeped aroma hops
+   preinstructions.clear();
+   preinstructions += getHopSteps(Hop::USEAROMA);
+   addPreinstructions(preinstructions);
+   
    // Fermentation instructions
    preinstructions.clear();
 
    /*** Fermentables added after boil ***/
    ins = getPostboilFermentables();
    if ( ins )
-	   instructions += ins;
+      instructions += ins;
 
    /*** post boil ***/
    ins = getPostboilSteps();
    if ( ins )
-	   instructions += ins;
+      instructions += ins;
    
    /*** Primary yeast ***/
    str = QObject::tr("Cool wort and pitch ");
@@ -1412,6 +1436,7 @@ void Recipe::addFermentable( Fermentable* var )
       hasChanged();
    }
 }
+
 void Recipe::addMisc( Misc* var )
 {
    if( var == NULL )
@@ -1448,6 +1473,18 @@ void Recipe::addWater( Water* var )
    }
 }
 
+void Recipe::addBrewNote(BrewNote* var)
+{
+   if ( var == NULL ) 
+      Brewtarget::logW( QString("Recipe: null brewnote"));
+   else 
+   {
+      brewNotes.push_back(var);
+      addObserved(var);
+      hasChanged();
+   }
+}
+
 void Recipe::setMash( Mash *var )
 {
    if( var == NULL )
@@ -1458,7 +1495,6 @@ void Recipe::setMash( Mash *var )
    hasChanged(QVariant(MASH));
 }
 
-
 void Recipe::setAsstBrewer( const QString &var )
 {
    asstBrewer = QString(var);
@@ -1822,6 +1858,22 @@ Water* Recipe::getWater(unsigned int i)
       return waters[i];
 }
 
+unsigned int Recipe::getNumBrewNotes() const
+{
+   return brewNotes.size();
+}
+
+BrewNote* Recipe::getBrewNote(unsigned int i)
+{
+   if ( static_cast<int>(i) >= brewNotes.size())
+   {
+      Brewtarget::logW( QString("Recipe: bad index into brewnotes: %1").arg(i) );
+      return 0;
+   }
+   else
+      return brewNotes[i];
+}
+
 Mash* Recipe::getMash() const
 {
    return mash;
@@ -2182,6 +2234,39 @@ bool Recipe::removeYeast(Yeast* var)
    return false;
 }
 
+bool Recipe::removeBrewNote(BrewNote* var)
+{
+   QVector<BrewNote*>::iterator iter;
+
+   for( iter = brewNotes.begin(); iter != brewNotes.end(); iter++ )
+   {
+      if( *iter == var )
+      {
+         brewNotes.erase(iter);
+         hasChanged();
+         return true;
+      }
+   }
+
+   return false;
+}
+
+bool Recipe::removeBrewNote(QList<BrewNote*> var)
+{
+   QVector<BrewNote*>::iterator iter;
+
+   for( iter = brewNotes.begin(); iter != brewNotes.end(); iter++ )
+   {
+      if( var.contains(*iter) )
+      {
+         brewNotes.erase(iter);
+         hasChanged();
+         return true;
+      }
+   }
+   return false;
+}
+
 double Recipe::getBoilGrav()
 {
    unsigned int i;
@@ -2217,6 +2302,34 @@ double Recipe::getBoilGrav()
    return Algorithms::Instance().PlatoToSG_20C20C( Algorithms::Instance().getPlato(sugar_kg, estimateBoilVolume_l()) );
 }
 
+// Return the theoretical maximum yield without any non-mashed anything.  This
+// will need to be communicated somewhere.
+double Recipe::getPoints(double volume)
+{
+   unsigned int i;
+   Fermentable* ferm;
+   double sugar_kg = 0.0;
+   double sugar_kg_ignoreEfficiency = 0.0;
+   Fermentable::Type type;
+
+   // Calculate OG
+   for( i = 0; static_cast<int>(i) < fermentables.size(); ++i )
+   {
+      ferm = fermentables[i];
+      if( ferm->getAddAfterBoil() )
+         continue;
+
+      // If we have some sort of non-grain, we have to ignore efficiency.
+      type = ferm->getType();
+      if( type==Fermentable::TYPESUGAR || type==Fermentable::TYPEEXTRACT || type==Fermentable::TYPEDRY_EXTRACT )
+         sugar_kg_ignoreEfficiency += (ferm->getYield_pct()/100.0)*ferm->getAmount_kg();
+      else
+         sugar_kg += (ferm->getYield_pct()/100.0)*ferm->getAmount_kg();
+   }
+
+   return 1000 * (  Algorithms::Instance().PlatoToSG_20C20C( Algorithms::Instance().getPlato(sugar_kg,volume)) - 1);
+}
+
 double Recipe::getIBU()
 {
    unsigned int i;
